/*
 * MultiUnzipView.java
 */

package multiunzip;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.EventObject;
import java.util.Properties;
import java.util.Vector;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import org.jdesktop.application.Application;


/**
 * The application's main frame.
 */
public class MultiUnzipView extends FrameView {

    public static class MyExitListener implements  Application.ExitListener
    {
        
        MultiUnzipView view;
        
        public MyExitListener( MultiUnzipView view )
        {
            this.view = view;
        }
                        
         public boolean canExit(EventObject e) 
         {
            view.saveProps();
            return true;
         }
         
        public void willExit(EventObject event) {}
        
    };
    
    public MultiUnzipView(SingleFrameApplication app) {
        super(app);                        

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
        
        initTree( TreeIn );
        initTree( TreeOut );
        initList( ListIn );
        initList( ListOut );
       
        System.out.println("os: " + System.getProperty("os.name"));
        
        config_name = resourceMap.getString("Application.name") + ".properties";
        
        config_path = System.getProperty("user.home");
        
        if(!System.getProperty("os.name").matches(".*[Ww][Ii][Nn].*"))
        {
           config_name = "." + config_name;    
        }
        
        config_file = config_path + File.separator + config_name;
        
        System.out.println("config file: " + config_file);
        
        loadProps();
        
        getApplication().addExitListener( new MyExitListener(this) );
    }

    private void loadProps()
    {
        Properties props = new Properties();
        
        try {
            FileInputStream in = new FileInputStream(config_file);
            props.load(in);
            in.close();
        } catch(IOException ioe) {
        
            System.err.println("Unhandled exception:");
            ioe.printStackTrace();
            return;
        }               
        
        lastInPath = props.getProperty("lastInPath");
        lastOutPath = props.getProperty("lastOutPath");

        if( lastInPath == null )
        {
            lastInPath = new String();       
        }
        
        if( lastOutPath == null )
        {
            lastOutPath = new String();        
        }                                
        
        setTreePath( TreeIn, lastInPath );
        setTreePath( TreeOut, lastOutPath );
        listAllFiles(lastOutPath,ListOut);
        listZipFiles(lastInPath,ListIn);
        
    }
    
    private void saveProps()
    {
        Properties props = new Properties();
        
        props.setProperty("lastInPath", lastInPath);
        props.setProperty("lastOutPath", lastOutPath);
        
         try {
            FileOutputStream out = new FileOutputStream(config_file);
            props.store(out, "nix");
            out.close();
        } catch(IOException ioe) {
        
            System.err.println("Unhandled exception:");
            ioe.printStackTrace();
            return;
        }               

    }
    
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = MultiUnzipApp.getApplication().getMainFrame();
            aboutBox = new MultiUnzipAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        MultiUnzipApp.getApplication().show(aboutBox);
    }
    
    private void initTree( javax.swing.JTree  tree )
    {
       DefaultMutableTreeNode root = new DefaultMutableTreeNode("Computer");
       TreeModel treeModel = new DefaultTreeModel(root);
       
       tree.setModel(treeModel);
       
       File roots[] = File.listRoots();
       
       int i = 0;
       
       for( File f : roots)
       {
           String name = f.getAbsoluteFile().toString();
           DefaultMutableTreeNode node = new DefaultMutableTreeNode(name);
           root.insert(node, i++);
       }
    }

    private void initList( JList list )
    {
        list.setModel(new DataListModel());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        TreeIn = new javax.swing.JTree();
        jScrollPane2 = new javax.swing.JScrollPane();
        TreeOut = new javax.swing.JTree();
        jScrollPane3 = new javax.swing.JScrollPane();
        ListIn = new javax.swing.JList();
        jScrollPane4 = new javax.swing.JScrollPane();
        ListOut = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        TreeIn.setName("TreeIn"); // NOI18N
        TreeIn.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                TreeInValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(TreeIn);

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        TreeOut.setName("TreeOut"); // NOI18N
        TreeOut.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                TreeOutValueChanged(evt);
            }
        });
        jScrollPane2.setViewportView(TreeOut);

        jScrollPane3.setName("jScrollPane3"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(multiunzip.MultiUnzipApp.class).getContext().getResourceMap(MultiUnzipView.class);
        ListIn.setFont(resourceMap.getFont("ListIn.font")); // NOI18N
        ListIn.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        ListIn.setName("ListIn"); // NOI18N
        ListIn.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                ListInValueChanged(evt);
            }
        });
        jScrollPane3.setViewportView(ListIn);

        jScrollPane4.setName("jScrollPane4"); // NOI18N

        ListOut.setFont(resourceMap.getFont("ListOut.font")); // NOI18N
        ListOut.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        ListOut.setName("ListOut"); // NOI18N
        jScrollPane4.setViewportView(ListOut);

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setFont(resourceMap.getFont("jLabel3.font")); // NOI18N
        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jScrollPane2)
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 257, Short.MAX_VALUE))
                            .addComponent(jLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel3)
                            .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 267, Short.MAX_VALUE)
                            .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 267, Short.MAX_VALUE))))
                .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGap(17, 17, 17)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jScrollPane3)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 249, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jScrollPane4)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 230, Short.MAX_VALUE))
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(multiunzip.MultiUnzipApp.class).getContext().getActionMap(MultiUnzipView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 560, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 376, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

     protected void addChildren(MutableTreeNode node, String path) {
         
         if( path.isEmpty() )
             return;
         
        //create filter for directories only
        DirectoryFileFilter filter = new DirectoryFileFilter();
        
        //get directories
        File dir = new File(path);
        File subDirs[] = dir.listFiles(filter);
        
        //add sub nodes to given nodes
        if (subDirs != null) {
            int i = 0;
            for (File f : subDirs) {
                MutableTreeNode child = new DefaultMutableTreeNode(f.getName());
                Boolean found = false;
                
                for( int j = 0; j < node.getChildCount(); j++ )
                {
                   if( node.getChildAt(j).toString().equals(f.getName()))
                   {
                       found = true;
                       break;
                   }
                }
                
                if( !found )
                    node.insert(child, i++);
            }
        }
    }
     
     protected String escape( String str )
     {
         return str.replace("\\", "\\\\");                  
     }
     
     protected Boolean isWin()
     {
        return System.getProperty("os.name").matches(".*[Ww][Ii][Nn].*");
     }
     
     protected void setTreePath( JTree tree, String path )
     {   
         //tree.setExpandsSelectedPaths(true);
         
         
         if( path.isEmpty() )
             return;                   
          
         String[] subdirs = path.split( escape(File.separator) );
         
         for( String s : subdirs )
         {
             System.out.println( "subdir: " + s );
         }
         
         DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getModel().getRoot();
         
         Vector<String> sd = new Vector<String>();
         
         for( String s : subdirs )
         {
             if( s.isEmpty() )                      
             {
                if( !isWin())
                    s = File.separator;
                else
                    continue;
             } 
             
             if( !sd.isEmpty() )
             {
                if( sd.lastElement().equals(File.separator) && 
                        s.equals(File.separator))
                     continue;
             }
                   
             if( isWin() && s.endsWith(":"))
             {
                 s += "\\";
             }
             
            sd.add(s); 
         }
         
         DefaultMutableTreeNode n = addPath( node, sd, 0 );
         
         
         DefaultTreeModel m = (DefaultTreeModel) tree.getModel();
         
         TreePath p = new TreePath(m.getPathToRoot(n));
         System.out.println("path: " + p.toString() +
                 " count " + p.getPathCount()
          + " parent " + p.getParentPath());
         
         System.out.println( "isleaf: " +
                 m.isLeaf(p.getLastPathComponent()));
         
         if( p.getParentPath() != null )
         {
            System.out.println( "parent isleaf: " +
                     m.isLeaf(p.getParentPath().getLastPathComponent()));
         }
         
         if(m.isLeaf(p.getLastPathComponent()))
         {
             p = p.getParentPath();
         }
         
         tree.expandPath(p);
         tree.scrollPathToVisible(p);
         tree.setLeadSelectionPath(p);
     }
     
     protected String buildPath( Vector<String> sd, int idx )
     {
         String path = new String();
         
         for( int i = 0; i <= idx; i++ )
         {
             if( sd.elementAt(i).equals(File.separator))
             {
                 if( !path.equals(File.separator))
                    path += sd.elementAt(i);
             } else {
                if( path.endsWith(File.separator))
                    path += sd.elementAt(i);
                else
                    path += File.separator + sd.elementAt(i);
             }
         }
         
         System.out.println(path);
         
         return path;
     }

     protected DefaultMutableTreeNode addPath( DefaultMutableTreeNode node, Vector<String> sd, int idx )
     {                     
         if( sd.size() <= idx )
         {
             if( node.getChildCount() == 0 &&
                 idx > 0 )
             {
                 addChildren( node, buildPath( sd, idx - 1 ));
             }
             return node;   
         }
         
         if( node.getChildCount() == 0 )
         {
             addChildren( node, buildPath( sd, idx - 1 ) );
         }                  
         
         for( int i = 0 ; i < node.getChildCount(); i++ )
         {
             DefaultMutableTreeNode sn = (DefaultMutableTreeNode) node.getChildAt(i);

             if( sn.toString().equals(sd.elementAt(idx)) )
             {                                       
                 return addPath( sn, sd, ++idx );
             }             
          }
                           
         
//          addChildren( node, buildPath( sd, idx ) );
//         addPath( node, sd, ++idx );
         return node;
     }
     
     protected void listAllFiles( String path, JList list )
     {        
        listFiles( path, list, new NormalFileFilter() );
     }
     
     protected void listZipFiles( String path, JList list )
     {        
        listFiles( path, list, new ZipFileFilter() );
     }
     
     protected void listFiles( String path, JList list, FileFilter filter )
     {
     
        //FileFilter filter = new ZipFileFilter();
        
        //get directories
        File dir = new File(path);
        File files[] = dir.listFiles(filter); 
        
         if( files == null )
         {
             return;
         }
        
        Arrays.sort( files, new Comparator()
        {
            public int compare(Object o1, Object o2) {
     	
    		if (((File)o1).lastModified() > ((File)o2).lastModified()) {
    			return -1;
    		} else if (((File)o1).lastModified() < ((File)o2).lastModified()) {
    			return +1;
    		} else {
    			return 0;
    		}
            }
    
        }); 
        
        DataListModel model = (DataListModel)list.getModel();
        
        if( model != null )
        {
            model.clear();
        }
        
        if( files != null )
        {
            for( File f : files )
            {
               String n =  f.getName();
               
               for( int i = n.length(); i < 40; i++ )
               {
                   n += " ";
               }
                
               model.addElement( n + " " + new Date(f.lastModified()), f.getAbsolutePath() );
            }
        }
     }
    
    private String TreeValueChanged(javax.swing.event.TreeSelectionEvent evt,
            JList list, FileFilter filter ) {                      
        
        TreePath path = evt.getNewLeadSelectionPath();
        
        if( path == null )
            return null;
        
         //get the selected node
         DefaultMutableTreeNode curNode = (DefaultMutableTreeNode) path.getLastPathComponent();
            
         //get all the nodes in the selected tree path
         Object nodes[] = path.getPath();
         String dir = "";
            
         //loop through all the tree nodes getting the directory name
         //we start from node one, since node 0, the top node, is the Computer node,
         //which will have no meaning in the file system path
         for (int i = 1; i < nodes.length; i++) {
             DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodes[i];
             //the user object will hold the directory name
             dir += (String)node.getUserObject();
             //add the platform independent directory separator character
             dir += File.separator;
         }
            
          //add the sub directory nodes to the selected one
          addChildren(curNode, dir);          
          
          listFiles(dir, list, filter );
          
          return dir;
    }
    
    private String getTreePath( JTree tree )
    {
        TreePath path = tree.getLeadSelectionPath();
        
        if( path == null )
            return null;
        
                 //get the selected node
         DefaultMutableTreeNode curNode = (DefaultMutableTreeNode) path.getLastPathComponent();
            
         //get all the nodes in the selected tree path
         Object nodes[] = path.getPath();
         String dir = "";
            
         //loop through all the tree nodes getting the directory name
         //we start from node one, since node 0, the top node, is the Computer node,
         //which will have no meaning in the file system path
         for (int i = 1; i < nodes.length; i++) {
             DefaultMutableTreeNode node = (DefaultMutableTreeNode) nodes[i];
             //the user object will hold the directory name
             dir += (String)node.getUserObject();
             //add the platform independent directory separator character
             dir += File.separator;
         }
         
         return dir;
    }
     
    private void TreeInValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_TreeInValueChanged
       String path = TreeValueChanged( evt, ListIn, new ZipFileFilter() );       
       
       if( path != null )
            lastInPath = path;       
    }//GEN-LAST:event_TreeInValueChanged

    private void TreeOutValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_TreeOutValueChanged
       String path = TreeValueChanged( evt, ListOut, new NormalFileFilter() );
                
       if( path != null )           
            lastOutPath = path;
    }//GEN-LAST:event_TreeOutValueChanged

    private void ListInValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_ListInValueChanged
        // TODO add your handling code here:
        if( evt.getValueIsAdjusting() == true )
            return;
        
        int idx = ListIn.getSelectedIndex();
        
        if( idx == -1 )
            return;
                
        DataListModel model = (DataListModel) ListIn.getModel();
        
        String name = (String) model.get(idx);
        String abs_name = (String) model.getData(idx);
        String fname = abs_name.substring(abs_name.lastIndexOf(File.separator));
        
        
        // System.out.println( abs_name );
        String path = getTreePath( TreeOut );
        
        if( path == null || path.isEmpty() )
        {
            JOptionPane.showMessageDialog(null, 
                        "Please select a output Path first!", 
                        "Error",
                        JOptionPane.OK_OPTION);
        }
        else
        {
        //    java.beans.PropertyChangeEvent ev = new java.beans.PropertyChangeEvent(this, "started", name, name);
            
            
            
            System.out.println( path );
            if( !UnzipFile.unzipFile(abs_name, path) )
            {
               statusMessageLabel.setText("extracting " + fname + 
                            " to directory " + path + "failed"  );
            } else {
               statusMessageLabel.setText("extracting " + fname + 
                            " to " + path + "succeeded"  );
               listAllFiles( path, ListOut );               
            }
        }
    }//GEN-LAST:event_ListInValueChanged

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList ListIn;
    private javax.swing.JList ListOut;
    private javax.swing.JTree TreeIn;
    private javax.swing.JTree TreeOut;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
    private String lastOutPath = new String();
    private String lastInPath = new String();
    private String config_name = null;
    private String config_path = null;
    private String config_file = null;
}
